//+------------------------------------------------------------------+
//|                                                    TimerDemo.mq4 |
//|                                                Maxim A.Kuznetsov |
//|                                                       nektomk.ru |
//+------------------------------------------------------------------+
#property copyright "Maxim A.Kuznetsov"
#property link      "nektomk.ru"
#property version   "1.00"
#property strict

#include "Timer.mqh"
#include "Timer.mq4"
//--- input parameters
input int      resolution=1;  // resolution

input int      sec1=3;        // FIRST timer period
input int      delay1=0;      // FIRST timer delay (rel. startTime)
input int      limit1=0;      // FIRST timer limit

input int      sec2=5;        // SECOND timer period
input int      delay2=0;      // SECOND time delay (rel. curr TF)
input int      limit2=0;      // SECOND limit

input int      sec3=8;        // THIRD period
input int      delay3=0;      // THIRD delay (rel. M1)
input int      limit3=0;      // THIRD limit

class MyTimer:public Timer {
public:
   string name;
   int count;
   int limit;
public:
   MyTimer(string _name,int _limit,int _period,datetime _startTime=0):Timer(_period,_startTime) {
      name=_name;
      limit=_limit;
      count=0;
      PrintFormat("Timer %s period=%d constructed",name,period);
   }
   ~MyTimer() {
      PrintFormat("Timer %s period=%d destroyed",name,period);
   }
   bool MyTimer::Clock() {
      if (count==0) 
         PrintFormat("Timer %s period=%d FIRSTCALL omit=%d lag=%d",name,period,Omit(),Lag());
      else
         PrintFormat("Timer %s period=%d CALL %d omit=%d lag=%d",name,period,count,Omit(),Lag());
      count++;
      count+=Omit();   
      if (limit!=0 && count>=limit) {
         PrintFormat("Timer %s period=%d STOP",name,period);
         return false;
      }
      return true;
   }
   void MyTimer::OnAttach() {
      PrintFormat("Timer %s with period=%d Attached",name,period);
   }
   void MyTimer::OnDetach() {
      PrintFormat("Timer %s with period=%d Detached",name,period);
   }
};
MyTimer *timer[3];
TimeService *ts;
int OnInit()
{  
   EventSetTimer(resolution);
   timer[0]=new MyTimer("FIRST",limit1,sec1,TimeCurrent()+delay1);
   timer[1]=new MyTimer("SECOND",limit2,sec2,iTime(_Symbol,_Period,0)+delay2);
   timer[2]=new MyTimer("THIRD",limit3,sec3,iTime(_Symbol,PERIOD_M1,0)+delay3);

   ts=new TimeService();
   ts.Add(timer[0]);      
   ts.Add(timer[1]);      
   ts.Add(timer[2]);      
   return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason)
{
   EventKillTimer();
   delete ts;
   delete timer[0];
   delete timer[1];
   delete timer[2];
}

void OnTick()
{
   if (IsTesting()) ts.Clock();  //    
}
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
{
   ts.Clock();
}
//+------------------------------------------------------------------+
